require 'helper'
require 'tight-engine/defer'

describe Tight::Engine::Defer do
  include Tight::Engine::Defer

  def process_element(*args)
    "block test\n" + args.inspect + @iteration
  end

  def args
    ['Block', [1, 3], :false => true]
  end

  before do
    @iteration = '1'
    settings.clear
    placeholders.clear
    engine.clear
  end

  it 'should have no initial placeholders' do
    assert_equal({}, placeholders)
  end

  it 'should defer elements set to be deferred' do
    settings[:deferred_elements] = ['Block']

    text = defer_element(*args)

    assert_equal({'Block' => args}, placeholders)
    assert_equal "%{placeholder[:Block]}", text
  end

  it 'should process deferred elements' do
    settings[:deferred_elements] = ['Block']

    text = defer_element(*args)
    @iteration = '2'
    result = process_deferred_elements('before' + text + 'after')

    assert_equal("beforeblock test\n" + args.inspect + '2after', result)
  end

  it 'should not defer any elements by default and process them right away' do
    text = defer_element(*args)
    @iteration = '2'
    result = process_deferred_elements('before' + text + 'after')

    assert_equal({}, placeholders)
    assert_equal("block test\n" + args.inspect + '1', text)
    assert_equal("beforeblock test\n" + args.inspect + '1after', result)
  end

  it 'should be able to be pretty' do
    settings[:deferred_elements] = ['Block']
    engine[:pretty?] = true

    text = defer_element(*args)
    @iteration = '2'
    result = process_deferred_elements('  ' + text)

    assert_equal("  block test\n  " + args.inspect + "2\n  ", result)
  end
end
